@@ -64,9 +64,6 @@ static Cipher Ciphers[] =
 	{ SERPENT,	L"Serpent",		16,			32,			140*4				},
 	{ TWOFISH,	L"Twofish",		16,			32,			TWOFISH_KS			},
 	{ CAMELLIA,	L"Camellia",	16,			32,			CAMELLIA_KS			},
-#if defined(CIPHER_GOST89)
-	{ GOST89,	L"GOST89",		16,			32,			GOST_KS },
-#endif  // defined(CIPHER_GOST89)
 	{ KUZNYECHIK,	L"Kuznyechik",16,		32,			KUZNYECHIK_KS },
 #endif
 	{ 0,		0,				0,			0,			0					}
@@ -85,9 +82,6 @@ static EncryptionAlgorithm EncryptionAlgorithms[] =
 	{ { SERPENT,					0 }, { XTS, 0 },	1, 1 },
 	{ { TWOFISH,					0 }, { XTS, 0 },	1, 1 },
 	{ { CAMELLIA,					0 }, { XTS, 0 },	1, 1 },
-#if defined(CIPHER_GOST89)
-	{ { GOST89,						0 }, { XTS, 0 },	0, 0 },
-#endif  // defined(CIPHER_GOST89)
 	{ { KUZNYECHIK,				0 }, { XTS, 0 },	0, 1 },
 	{ { TWOFISH, AES,				0 }, { XTS, 0 },	1, 1 },
 	{ { SERPENT, TWOFISH, AES,	0 }, { XTS, 0 },	1, 1 },
@@ -126,8 +120,8 @@ static Hash Hashes[] =
 {	// ID				Name					Deprecated	System Encryption
 	{ SHA512,		L"SHA-512",				FALSE,	FALSE },
 	{ WHIRLPOOL,	L"Whirlpool",			FALSE,	FALSE },
+	{ BLAKE2S,		L"BLAKE2s-256",				FALSE,	TRUE },
 	{ SHA256,		L"SHA-256",				FALSE,	TRUE },
-	{ RIPEMD160,	L"RIPEMD-160",			TRUE,		TRUE },
 	{ STREEBOG,		L"Streebog",	FALSE,	FALSE },
 	{ 0, 0, 0 }
 };
@@ -168,11 +162,6 @@ int CipherInit (int cipher, unsigned char *key, unsigned __int8 *ks)
 #endif
 
 #if !defined(TC_WINDOWS_BOOT) 
-#if defined(CIPHER_GOST89)
-	case GOST89:
-		gost_set_key(key, (gost_kds*)ks, 1);
-		break;
-#endif // && defined(CIPHER_GOST89)
 	case KUZNYECHIK:
 		kuznyechik_set_key(key, (kuznyechik_kds*)ks);
 		break;
@@ -206,9 +195,6 @@ void EncipherBlock(int cipher, void *data, void *ks)
 	case CAMELLIA:		camellia_encrypt (data, data, ks); break;
 #endif
 #if !defined(TC_WINDOWS_BOOT)
-#if defined(CIPHER_GOST89)
-	case GOST89:		gost_encrypt(data, data, ks, 1); break;
-#endif // defined(CIPHER_GOST89)
 	case KUZNYECHIK:		kuznyechik_encrypt_block(data, data, ks); break;
 #endif // !defined(TC_WINDOWS_BOOT) 
 	default:			TC_THROW_FATAL_EXCEPTION;	// Unknown/wrong ID
@@ -281,9 +267,6 @@ void EncipherBlocks (int cipher, void *dataPtr, void *ks, size_t blockCount)
 #endif
 	}
 #endif
-	else if (cipher == GOST89)	{
-			gost_encrypt(data, data, ks, (int)blockCount);
-	}
 	else
 	{
 		size_t blockSize = CipherGetBlockSize (cipher);
@@ -307,9 +290,6 @@ void DecipherBlock(int cipher, void *data, void *ks)
 	case CAMELLIA:	camellia_decrypt (data, data, ks); break;
 #endif
 #if !defined(TC_WINDOWS_BOOT)
-#if defined(CIPHER_GOST89)
-	case GOST89:	gost_decrypt(data, data, ks, 1); break;
-#endif // defined(CIPHER_GOST89)
 	case KUZNYECHIK:	kuznyechik_decrypt_block(data, data, ks); break;
 #endif // !defined(TC_WINDOWS_BOOT)
 
@@ -398,9 +378,6 @@ void DecipherBlocks (int cipher, void *dataPtr, void *ks, size_t blockCount)
 #endif
 	}
 #endif
-	else if (cipher == GOST89)	{
-			gost_decrypt(data, data, ks, (int)blockCount);
-	}
 	else
 	{
 		size_t blockSize = CipherGetBlockSize (cipher);
@@ -466,7 +443,6 @@ int CipherGetKeyScheduleSize (int cipherId)
 BOOL CipherSupportsIntraDataUnitParallelization (int cipher)
 {
 	return (cipher == AES && IsAesHwCpuSupported()) 
-		|| (cipher == GOST89)
 #if CRYPTOPP_BOOL_SSE2_INTRINSICS_AVAILABLE && !defined (_UEFI)
 		|| (cipher == SERPENT && HasSSE2())
 		|| (cipher == KUZNYECHIK && HasSSE2())
@@ -1204,7 +1180,7 @@ BOOL IsAesHwCpuSupported ()
 	}
 
 	return state && !HwEncryptionDisabled;
-#elif defined (_M_ARM64)
+#elif defined (_M_ARM64) || defined(__arm__) || defined (__arm64__) || defined (__aarch64__)
 	return 0;
 #else
 	return (HasAESNI() && !HwEncryptionDisabled)? TRUE : FALSE;
@@ -1329,8 +1305,8 @@ BOOL InitializeSecurityParameters(GetRandSeedFn rngCallback)
 	ChaCha20RngCtx ctx;
 	byte pbSeed[CHACHA20RNG_KEYSZ + CHACHA20RNG_IVSZ];
 #ifdef TC_WINDOWS_DRIVER
-	byte i, tagLength;
-
+	byte i;
+	char randomStr[4];
 	Dump ("InitializeSecurityParameters BEGIN\n");
 #endif
 
@@ -1339,28 +1315,57 @@ BOOL InitializeSecurityParameters(GetRandSeedFn rngCallback)
 	ChaCha20RngInit (&ctx, pbSeed, rngCallback, 0);
 
 #ifdef TC_WINDOWS_DRIVER
-	/* generate random tag length between 1 and 4 */
-	tagLength = GetRandomIndex (&ctx, 4) + 1;
 
-	/* generate random value for tag:
-	 * Each ASCII character in the tag must be a value in the range 0x20 (space) to 0x7E (tilde)
-	 * So we have 95 possibility
+	/* Generate random value for tag that is similar to pool tag values used by Windows kernel.
+	 * Fully random tag would be too suspicious and outstanding.
+     * First character is always a capital letter.
+     * Second character is a letter, lowercase or uppercase.
+     * Third character is a letter, lowercase or uppercase.
+     * Fourth character is a letter or a digit.
 	 */
+
+    /* 1. First character (Capital Letter) */
+    randomStr[0] = 'A' + GetRandomIndex(&ctx, 26);
+
+    /* 2. Second character (Letter) */
+    i = GetRandomIndex(&ctx, 52);
+    if (i < 26)
+        randomStr[1] = 'A' + i;
+    else
+        randomStr[1] = 'a' + (i - 26);
+
+    /* 3. Third character (Letter) */
+    i = GetRandomIndex(&ctx, 52);
+    if (i < 26)
+        randomStr[2] = 'A' + i;
+    else
+        randomStr[2] = 'a' + (i - 26);
+
+    /* 4. Fourth character (Letter or Digit) */
+    i = GetRandomIndex(&ctx, 62);
+    if (i < 26)
+        randomStr[3] = 'A' + i;
+    else if (i < 52)
+        randomStr[3] = 'a' + (i - 26);
+    else
+        randomStr[3] = '0' + (i - 52);
+
+	/* combine all characters in reverse order as explained in MSDN */
 	AllocTag = 0;
-	for (i = 0; i < tagLength; i++)
+	for (i = 0; i < 4; i++)
 	{
-		AllocTag = (AllocTag << 8) + (((ULONG) GetRandomIndex (&ctx, 95)) + 0x20);
+		AllocTag = (AllocTag << 8) + randomStr[3-i];
 	}
 
 #endif
 
 	cbKeyDerivationArea = 1024 * 1024;
-	pbKeyDerivationArea = (byte*) TCalloc(cbKeyDerivationArea);
-	if (!pbKeyDerivationArea)
+	do
 	{
-		cbKeyDerivationArea = 2 * PAGE_SIZE;
 		pbKeyDerivationArea = (byte*) TCalloc(cbKeyDerivationArea);
-	}
+		if (!pbKeyDerivationArea)
+			cbKeyDerivationArea >>= 1;
+	} while (!pbKeyDerivationArea && (cbKeyDerivationArea >= (2*PAGE_SIZE)));
 
 	if (!pbKeyDerivationArea)
 	{
@@ -1381,7 +1386,7 @@ BOOL InitializeSecurityParameters(GetRandSeedFn rngCallback)
 	FAST_ERASE64 (pbSeed, sizeof (pbSeed));
 	burn (&ctx, sizeof (ctx));
 #ifdef TC_WINDOWS_DRIVER
-	burn (&tagLength, 1);
+	burn (randomStr, sizeof(randomStr));
 
 	Dump ("InitializeSecurityParameters return=TRUE END\n");
 #endif
@@ -1426,11 +1431,20 @@ void VcProtectMemory (uint64 encID, unsigned char* pbData, size_t cbData,
 		hashLow = t1ha2_atonce128(&hashHigh, pbKeyDerivationArea, cbKeyDerivationArea, hashSeed);
 
 		/* set the key to the hash result */
-		pbKey[0] = pbKey[2] = hashLow;
-		pbKey[1] = pbKey[3] = hashHigh;
+		pbKey[0] = hashLow;
+		pbKey[1] = hashHigh;
+		/* we now have a 128-bit key and we will expand it to 256-bit by using ChaCha12 cipher */
+		/* first we need to generate a the other 128-bit half of the key */
+		pbKey[2] = hashLow ^ hashHigh;
+		pbKey[3] = hashLow + hashHigh;
 
 		/* Initialize ChaCha12 cipher */
-		cipherIV = encID ^ CipherIVMask;
+		ChaCha256Init (&ctx, (unsigned char*) pbKey, (unsigned char*) &hashSeed, 12);
+		/* encrypt the key by itself */
+		ChaCha256Encrypt (&ctx, (unsigned char*) pbKey, sizeof(pbKey), (unsigned char*) pbKey);
+
+		/* Initialize ChaCha12 cipher */
+		cipherIV = (((uint64) pbKeyDerivationArea) + encID) ^ CipherIVMask;
 		ChaCha256Init (&ctx, (unsigned char*) pbKey, (unsigned char*) &cipherIV, 12);
 
 		ChaCha256Encrypt (&ctx, pbData, cbData, pbData);
@@ -1487,7 +1501,7 @@ void VcUnprotectKeys (PCRYPTO_INFO pCryptoInfo, uint64 encID)
 
 #endif
 
-#ifdef _M_ARM64
+#if defined(_M_ARM64) || defined(__arm__) || defined (__arm64__) || defined (__aarch64__)
 /* dummy implementation that should never be called */
 void aes_hw_cpu_decrypt(const byte* ks, byte* data)
 {
