@@ -18,7 +18,7 @@
 
 namespace VeraCrypt
 {
-	VolumePasswordPanel::VolumePasswordPanel (wxWindow* parent, MountOptions* options, shared_ptr <VolumePassword> password, bool disableTruecryptMode, shared_ptr <KeyfileList> keyfiles, bool enableCache, bool enablePassword, bool enableKeyfiles, bool enableConfirmation, bool enablePkcs5Prf, bool isMountPassword, const wxString &passwordLabel)
+	VolumePasswordPanel::VolumePasswordPanel (wxWindow* parent, MountOptions* options, shared_ptr <VolumePassword> password, shared_ptr <KeyfileList> keyfiles, bool enableCache, bool enablePassword, bool enableKeyfiles, bool enableConfirmation, bool enablePkcs5Prf, bool isMountPassword, const wxString &passwordLabel)
 		: VolumePasswordPanelBase (parent), TopOwnerParent(NULL), Keyfiles (new KeyfileList), EnablePimEntry (true)
 	{
 		size_t maxPasswordLength = CmdLine->ArgUseLegacyPassword? VolumePassword::MaxLegacySize : VolumePassword::MaxSize;
@@ -78,22 +78,9 @@ namespace VeraCrypt
 
 		Pkcs5PrfStaticText->Show (enablePkcs5Prf);
 		Pkcs5PrfChoice->Show (enablePkcs5Prf);
-		TrueCryptModeCheckBox->Show (!disableTruecryptMode);
 		HeaderWipeCountText->Show (enablePkcs5Prf && !isMountPassword);
 		HeaderWipeCount->Show (enablePkcs5Prf && !isMountPassword);
 
-		if (options && !disableTruecryptMode)
-		{
-			TrueCryptModeCheckBox->SetValue (options->TrueCryptMode);
-			if (options->TrueCryptMode)
-			{
-				PimCheckBox->Enable (false);
-				VolumePimStaticText->Enable (false);
-				VolumePimTextCtrl->Enable (false);
-				VolumePimHelpStaticText->Enable (false);
-			}
-		}
-
 		if (EnablePimEntry && options && options->Pim > 0)
 		{
 			PimCheckBox->SetValue (true);
@@ -113,7 +100,7 @@ namespace VeraCrypt
 				Pkcs5PrfChoice->Delete (0);
 				Pkcs5PrfChoice->Append (LangString["AUTODETECTION"]);
 			}
-			foreach_ref (const Pkcs5Kdf &kdf, Pkcs5Kdf::GetAvailableAlgorithms(false))
+			foreach_ref (const Pkcs5Kdf &kdf, Pkcs5Kdf::GetAvailableAlgorithms())
 			{
 				if (!kdf.IsDeprecated() || isMountPassword)
 				{
@@ -204,7 +191,7 @@ namespace VeraCrypt
 		newTextCtrl->SetMinSize ((*textCtrl)->GetSize());
 
 		GridBagSizer->Detach ((*textCtrl));
-		GridBagSizer->Add (newTextCtrl, wxGBPosition (row, 1), wxGBSpan (1, colspan), wxEXPAND|wxBOTTOM|wxALIGN_CENTER_VERTICAL, 5);
+		GridBagSizer->Add (newTextCtrl, wxGBPosition (row, 1), wxGBSpan (1, colspan), wxEXPAND|wxBOTTOM, 5);
 		(*textCtrl)->Show (false);
 		WipeTextCtrl (*textCtrl);
 
@@ -221,7 +208,7 @@ namespace VeraCrypt
 
 	shared_ptr <VolumePassword> VolumePasswordPanel::GetPassword (bool bForceLegacyPassword) const
 	{
-		return GetPassword (PasswordTextCtrl, bForceLegacyPassword || GetTrueCryptMode());
+		return GetPassword (PasswordTextCtrl, bForceLegacyPassword);
 	}
 
 	shared_ptr <VolumePassword> VolumePasswordPanel::GetPassword (wxTextCtrl *textCtrl, bool bLegacyPassword) const
@@ -246,14 +233,8 @@ namespace VeraCrypt
 		return password;
 	}
 
-	shared_ptr <Pkcs5Kdf> VolumePasswordPanel::GetPkcs5Kdf (bool &bUnsupportedKdf) const
+	shared_ptr <Pkcs5Kdf> VolumePasswordPanel::GetPkcs5Kdf () const
 	{
-		return GetPkcs5Kdf (GetTrueCryptMode(), bUnsupportedKdf);
-	}
-
-	shared_ptr <Pkcs5Kdf> VolumePasswordPanel::GetPkcs5Kdf (bool bTrueCryptMode, bool &bUnsupportedKdf) const
-	{
-		bUnsupportedKdf = false;
 		try
 		{
 			int index = Pkcs5PrfChoice->GetSelection ();
@@ -263,11 +244,10 @@ namespace VeraCrypt
 				return shared_ptr <Pkcs5Kdf> ();
 			}
 			else
-				return Pkcs5Kdf::GetAlgorithm (wstring (Pkcs5PrfChoice->GetStringSelection()), bTrueCryptMode);
+				return Pkcs5Kdf::GetAlgorithm (wstring (Pkcs5PrfChoice->GetStringSelection()));
 		}
 		catch (ParameterIncorrect&)
 		{
-			bUnsupportedKdf = true;
 			return shared_ptr <Pkcs5Kdf> ();
 		}
 	}
@@ -303,21 +283,6 @@ namespace VeraCrypt
 		}
 	}
 
-	bool VolumePasswordPanel::GetTrueCryptMode () const
-	{
-		return TrueCryptModeCheckBox->GetValue ();
-	}
-	
-	void VolumePasswordPanel::SetTrueCryptMode (bool trueCryptMode)
-	{
-		bool bEnablePIM = !trueCryptMode;
-		TrueCryptModeCheckBox->SetValue (trueCryptMode);
-		PimCheckBox->Enable (bEnablePIM);
-		VolumePimStaticText->Enable (bEnablePIM);
-		VolumePimTextCtrl->Enable (bEnablePIM);
-		VolumePimHelpStaticText->Enable (bEnablePIM);
-	}
-
 	int VolumePasswordPanel::GetHeaderWipeCount () const
 	{
 		try
@@ -383,7 +348,7 @@ namespace VeraCrypt
 			SecurityTokenKeyfilesDialog dialog (this);
 			if (dialog.ShowModal() == wxID_OK)
 			{
-				foreach (const SecurityTokenKeyfilePath &path, dialog.GetSelectedSecurityTokenKeyfilePaths())
+				foreach (const TokenKeyfilePath &path, dialog.GetSelectedSecurityTokenKeyfilePaths())
 				{
 					Keyfiles->push_back (make_shared <Keyfile> (wstring (path)));
 				}
@@ -459,7 +424,11 @@ namespace VeraCrypt
 
 	void VolumePasswordPanel::WipeTextCtrl (wxTextCtrl *textCtrl)
 	{
-		textCtrl->SetValue (wxString (L'X', textCtrl->GetLineLength(0)));
+		int txtLen = textCtrl->GetLineLength(0);
+		if (txtLen > 0)
+		{
+			textCtrl->SetValue (wxString (L'X', txtLen));
+		}
 		GetPassword (textCtrl);
 	}
 
@@ -472,9 +441,9 @@ namespace VeraCrypt
 			VolumePimHelpStaticText->SetLabel(LangString["PIM_CHANGE_WARNING"]);
 			guiUpdated = true;
 		}
-		if (!pimChanged && VolumePimHelpStaticText->GetForegroundColour() != *wxBLACK)
+		if (!pimChanged && VolumePimHelpStaticText->GetForegroundColour() != wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT))
 		{
-			VolumePimHelpStaticText->SetForegroundColour(*wxBLACK);
+			VolumePimHelpStaticText->SetForegroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
 			VolumePimHelpStaticText->SetLabel(LangString["IDC_PIM_HELP"]);
 			guiUpdated = true;
 		}
@@ -511,13 +480,4 @@ namespace VeraCrypt
 			layoutParent->Fit();
 		}
 	}
-
-	void VolumePasswordPanel::OnTrueCryptModeChecked( wxCommandEvent& event )
-	{
-		bool bEnablePIM = !GetTrueCryptMode ();
-		PimCheckBox->Enable (bEnablePIM);
-		VolumePimStaticText->Enable (bEnablePIM);
-		VolumePimTextCtrl->Enable (bEnablePIM);
-		VolumePimHelpStaticText->Enable (bEnablePIM);
-	}
 }
