@@ -16,7 +16,6 @@
 #include "Crypto/SerpentFast.h"
 #include "Crypto/Twofish.h"
 #include "Crypto/Camellia.h"
-#include "Crypto/GostCipher.h"
 #include "Crypto/kuznyechik.h"
 
 #ifdef TC_AES_HW_CPU
@@ -98,7 +97,6 @@ namespace VeraCrypt
 		l.push_back (shared_ptr <Cipher> (new CipherSerpent ()));
 		l.push_back (shared_ptr <Cipher> (new CipherTwofish ()));
 		l.push_back (shared_ptr <Cipher> (new CipherCamellia ()));
-		l.push_back (shared_ptr <Cipher> (new CipherGost89 ()));
 		l.push_back (shared_ptr <Cipher> (new CipherKuznyechik ()));
 
 		return l;
@@ -399,48 +397,6 @@ namespace VeraCrypt
 #endif
 	}
 
-	// GOST89
-	void CipherGost89::Decrypt (byte *data) const
-	{
-		gost_decrypt (data, data, (gost_kds *) ScheduledKey.Ptr(), 1);
-	}
-
-	void CipherGost89::Encrypt (byte *data) const
-	{
-		gost_encrypt (data, data, (gost_kds *) ScheduledKey.Ptr(), 1);
-	}
-
-	size_t CipherGost89::GetScheduledKeySize () const
-	{
-		return GOST_KS;
-	}
-
-	void CipherGost89::SetCipherKey (const byte *key)
-	{
-		gost_set_key (key, (gost_kds *) ScheduledKey.Ptr(), 1);
-	}
-	
-	// GOST89 with static SBOX
-	void CipherGost89StaticSBOX::Decrypt (byte *data) const
-	{
-		gost_decrypt (data, data, (gost_kds *) ScheduledKey.Ptr(), 1);
-	}
-
-	void CipherGost89StaticSBOX::Encrypt (byte *data) const
-	{
-		gost_encrypt (data, data, (gost_kds *) ScheduledKey.Ptr(), 1);
-	}
-
-	size_t CipherGost89StaticSBOX::GetScheduledKeySize () const
-	{
-		return GOST_KS;
-	}
-
-	void CipherGost89StaticSBOX::SetCipherKey (const byte *key)
-	{
-		gost_set_key (key, (gost_kds *) ScheduledKey.Ptr(), 0);
-	}
-
 	// Kuznyechik
 	void CipherKuznyechik::Decrypt (byte *data) const
 	{
