@@ -65,7 +65,6 @@ namespace VeraCrypt
 		l.push_back (shared_ptr <EncryptionAlgorithm> (new Serpent ()));
 		l.push_back (shared_ptr <EncryptionAlgorithm> (new Twofish ()));
 		l.push_back (shared_ptr <EncryptionAlgorithm> (new Camellia ()));
-		l.push_back (shared_ptr <EncryptionAlgorithm> (new GOST89 ()));
 		l.push_back (shared_ptr <EncryptionAlgorithm> (new Kuznyechik ()));
 		l.push_back (shared_ptr <EncryptionAlgorithm> (new AESTwofish ()));
 		l.push_back (shared_ptr <EncryptionAlgorithm> (new AESTwofishSerpent ()));
@@ -301,17 +300,6 @@ namespace VeraCrypt
 		SupportedModes.push_back (shared_ptr <EncryptionMode> (new EncryptionModeXTS ()));
 	}
 
-	
-	// GOST89
-	GOST89::GOST89 ()
-	{
-		Deprecated = true;
-
-		Ciphers.push_back (shared_ptr <Cipher> (new CipherGost89()));
-
-		SupportedModes.push_back (shared_ptr <EncryptionMode> (new EncryptionModeXTS ()));
-	}
-
 	// Kuznyechik
 	Kuznyechik::Kuznyechik ()
 	{
